/*
  Copyright (c) 2011 Alex Diener
  
  This software is provided 'as-is', without any express or implied
  warranty. In no event will the authors be held liable for any damages
  arising from the use of this software.
  
  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:
  
  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.
  
  Alex Diener adiener@sacredsoftware.net
*/

#include "convergence/AudioManager.h"
#include "convergence/DungeonDoorModel.h"
#include "convergence/DungeonDoorView.h"
#include "convergence/GraphicsConstants.h"
#include <stddef.h>
#include <stdlib.h>

struct vertex_p2f_c3ub {
	GLfloat position[2];
	GLubyte color[3];
};

DungeonDoorView * DungeonDoorView_create(compat_type(DungeonDoorModel *) modelPtr) {
	stemobject_create_implementation(DungeonDoorView, init, modelPtr)
}

static bool unlocked(void * sender, Atom eventID, void * eventData, void * context) {
	AudioManager_play("door_unlocked");
	return true;
}

void DungeonDoorView_init(compat_type(DungeonDoorView *) selfPtr, compat_type(DungeonDoorModel *) modelPtr) {
	DungeonDoorView * self = selfPtr;
	DungeonDoorModel * model = modelPtr;
	struct vertex_p2f_c3ub vertices[4];
	GLushort indexes[6] = {0, 1, 2, 2, 3, 0};
	unsigned int color;
	
	GameObjectView_init(self, modelPtr);
	self->dispose = DungeonDoorView_dispose;
	self->draw = DungeonDoorView_draw;
	
	self->sortOrder = -1;
	
	if (model->keyID < 0 || model->keyID >= KEY_COLOR_COUNT) {
		color = 0xFFFFFF;
	} else {
		color = KEY_COLORS[model->keyID];
	}
	vertices[0].position[0] = model->bounds.left;
	vertices[0].position[1] = model->bounds.bottom;
	vertices[0].color[0] = color >> 16 & 0xFF;
	vertices[0].color[1] = color >> 8 & 0xFF;
	vertices[0].color[2] = color & 0xFF;
	vertices[1].position[0] = model->bounds.right;
	vertices[1].position[1] = model->bounds.bottom;
	vertices[1].color[0] = color >> 16 & 0xFF;
	vertices[1].color[1] = color >> 8 & 0xFF;
	vertices[1].color[2] = color & 0xFF;
	vertices[2].position[0] = model->bounds.right;
	vertices[2].position[1] = model->bounds.top;
	vertices[2].color[0] = color >> 16 & 0xFF;
	vertices[2].color[1] = color >> 8 & 0xFF;
	vertices[2].color[2] = color & 0xFF;
	vertices[3].position[0] = model->bounds.left;
	vertices[3].position[1] = model->bounds.top;
	vertices[3].color[0] = color >> 16 & 0xFF;
	vertices[3].color[1] = color >> 8 & 0xFF;
	vertices[3].color[2] = color & 0xFF;
	
	glGenBuffers(1, &self->vboID);
	glGenBuffers(1, &self->eboID);
	glBindBuffer(GL_ARRAY_BUFFER, self->vboID);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, self->eboID);
	glBufferData(GL_ARRAY_BUFFER, sizeof(struct vertex_p2f_c3ub) * 4, vertices, GL_STATIC_DRAW);
	glBufferData(GL_ELEMENT_ARRAY_BUFFER, sizeof(GLushort) * 6, indexes, GL_STATIC_DRAW);
	glBindBuffer(GL_ARRAY_BUFFER, 0);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
	
	model->eventDispatcher->registerForEvent(model->eventDispatcher, Atom_fromString(DUNGEON_DOOR_EVENT_UNLOCKED), unlocked, self);
}

void DungeonDoorView_dispose(compat_type(DungeonDoorView *) selfPtr) {
	DungeonDoorView * self = selfPtr;
	DungeonDoorModel * model = (DungeonDoorModel *) self->model;
	
	model->eventDispatcher->unregisterForEvent(model->eventDispatcher, Atom_fromString(DUNGEON_DOOR_EVENT_UNLOCKED), unlocked, self);
	
	glDeleteBuffers(1, &self->vboID);
	glDeleteBuffers(1, &self->eboID);
	StemObject_dispose(self);
}

void DungeonDoorView_draw(compat_type(DungeonDoorView *) selfPtr, Matrix modelviewMatrix, double deltaTime) {
	DungeonDoorView * self = selfPtr;
	
	glBindBuffer(GL_ARRAY_BUFFER, self->vboID);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, self->eboID);
	glMatrixMode(GL_MODELVIEW);
	glLoadMatrixf(modelviewMatrix.m);
	glEnableClientState(GL_VERTEX_ARRAY);
	glEnableClientState(GL_COLOR_ARRAY);
	glVertexPointer(2, GL_FLOAT, sizeof(struct vertex_p2f_c3ub), (void *) offsetof(struct vertex_p2f_c3ub, position));
	glColorPointer(3, GL_UNSIGNED_BYTE, sizeof(struct vertex_p2f_c3ub), (void *) offsetof(struct vertex_p2f_c3ub, color));
	glDrawElements(GL_TRIANGLES, 6, GL_UNSIGNED_SHORT, 0);
	glDisableClientState(GL_VERTEX_ARRAY);
	glDisableClientState(GL_COLOR_ARRAY);
	glBindBuffer(GL_ARRAY_BUFFER, 0);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
}
