/*
  Copyright (c) 2011 Alex Diener
  
  This software is provided 'as-is', without any express or implied
  warranty. In no event will the authors be held liable for any damages
  arising from the use of this software.
  
  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:
  
  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.
  
  Alex Diener adiener@sacredsoftware.net
*/

#include "convergence/DungeonTileColorMap.h"
#include <stdio.h>
#include <stdlib.h>

DungeonTileColorMap * DungeonTileColorMap_create() {
	stemobject_create_implementation(DungeonTileColorMap, init)
}

void DungeonTileColorMap_init(compat_type(DungeonTileColorMap *) selfPtr) {
	DungeonTileColorMap * self = selfPtr;
	
	StemObject_init(self);
	self->dispose = DungeonTileColorMap_dispose;
	self->colorCount = 0;
	self->colors = NULL;
}

void DungeonTileColorMap_dispose(compat_type(DungeonTileColorMap *) selfPtr) {
	DungeonTileColorMap * self = selfPtr;
	
	free(self->colors);
	StemObject_dispose(self);
}

DungeonTileColorMap * DungeonTileColorMap_deserialize(compat_type(DeserializationContext *) deserializationContext) {
	stemobject_deserialize_implementation(DungeonTileColorMap, deserializationContext, loadSerializedData)
}

bool DungeonTileColorMap_loadSerializedData(compat_type(DungeonTileColorMap *) selfPtr, compat_type(DeserializationContext *) deserializationContext) {
	DungeonTileColorMap * self = selfPtr;
	DeserializationContext * context = deserializationContext;
	uint16_t formatVersion;
	size_t colorCount, colorIndex;
	struct DungeonTileColorMap_color * colors;
	const char * colorString;
	
	context->beginStructure(context, "color_map");
	formatVersion = context->readUInt16(context, "format_version");
	if (context->status == SERIALIZATION_ERROR_OK && formatVersion != DUNGEON_TILE_COLOR_MAP_SERIALIZATION_FORMAT_VERSION) {
		return false;
	}
	colorCount = context->beginDictionary(context, "colors");
	colors = malloc(sizeof(struct DungeonTileColorMap_color) * colorCount);
	for (colorIndex = 0; colorIndex < colorCount; colorIndex++) {
		colorString = context->readNextDictionaryKey(context);
		if (colorString == NULL) {
			break;
		}
		if (!sscanf(colorString, "%x", &colors[colorIndex].color)) {
			free(colors);
			return false;
		}
		context->beginStructure(context, colorString);
		colors[colorIndex].type = context->readEnumeration(context, "type",
			"wall", DUNGEON_TILE_TYPE_WALL,
			"floor", DUNGEON_TILE_TYPE_FLOOR,
		NULL);
		colors[colorIndex].canvasTileID = Atom_fromString(context->readString(context, "tile"));
		context->endStructure(context);
	}
	context->endDictionary(context);
	context->endStructure(context);
	
	if (context->status != SERIALIZATION_ERROR_OK) {
		free(colors);
		return false;
	}
	
	DungeonTileColorMap_init(self);
	self->colorCount = colorCount;
	self->colors = colors;
	
	return true;
}

void DungeonTileColorMap_serialize(compat_type(DungeonTileColorMap *) selfPtr, compat_type(SerializationContext *) serializationContext) {
	DungeonTileColorMap * self = selfPtr;
	SerializationContext * context = serializationContext;
	size_t colorIndex;
	char colorString[16];
	
	context->beginStructure(context, "color_map");
	context->writeUInt16(context, "format_version", DUNGEON_TILE_COLOR_MAP_SERIALIZATION_FORMAT_VERSION);
	context->beginDictionary(context, "colors");
	for (colorIndex = 0; colorIndex < self->colorCount; colorIndex++) {
		snprintf(colorString, 16, "0x%06X", self->colors[colorIndex].color);
		context->beginStructure(context, colorString);
		context->writeEnumeration(context, "type", self->colors[colorIndex].type,
			"wall", DUNGEON_TILE_TYPE_WALL,
			"floor", DUNGEON_TILE_TYPE_FLOOR,
		NULL);
		context->writeString(context, "tile", self->colors[colorIndex].canvasTileID);
		context->endStructure(context);
	}
	context->endDictionary(context);
	context->endStructure(context);
}
