/*
  Copyright (c) 2011 Alex Diener
  
  This software is provided 'as-is', without any express or implied
  warranty. In no event will the authors be held liable for any damages
  arising from the use of this software.
  
  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:
  
  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.
  
  Alex Diener adiener@sacredsoftware.net
*/

#ifndef __GAMEPLAY_STATE_H__
#define __GAMEPLAY_STATE_H__

typedef struct GameplayState GameplayState;

#include "convergence/DungeonModel.h"
#include "convergence/GameplayStateView.h"
#include "convergence/GameSession.h"
#include "convergence/GameStateController.h"
#include "convergence/GameStateModel.h"
#include "convergence/PauseScreenView.h"
#include "convergence/PlayerInputController.h"
#include "convergence/SharedDefinitions.h"
#include "glgraphics/GLIncludes.h"
#include "statecontroller/State.h"
#include "utilities/FixedIntervalRunLoop.h"

#define GameplayState_structContents \
	State_structContents \
	\
	GameSession * gameSession; \
	FixedIntervalRunLoop * runLoop; \
	GameStateModel * gameStateModel; \
	GameStateController * gameStateController; \
	PlayerInputController * playerInputController; \
	GameplayStateView * view; \
	PauseScreenView * pauseScreenView; \
	\
	double lastDrawTime; \
	bool playerDied; \
	double playerDeathTime; \
	bool dungeonComplete; \
	bool dungeonFadeInComplete; \
	double dungeonCompleteTime; \
	bool backgrounded; \
	bool paused; \
	int pauseItemIndex;

struct GameplayState {
	GameplayState_structContents
};

GameplayState * GameplayState_create(GameSession * gameSession);
void GameplayState_init(compat_type(GameplayState *) selfPtr, GameSession * gameSession);
void GameplayState_dispose(compat_type(GameplayState *) selfPtr);

void GameplayState_activate(compat_type(GameplayState *) selfPtr);
void GameplayState_deactivate(compat_type(GameplayState *) selfPtr);
void GameplayState_draw(compat_type(GameplayState *) selfPtr);

#endif
