/*
  Copyright (c) 2011 Alex Diener
  
  This software is provided 'as-is', without any express or implied
  warranty. In no event will the authors be held liable for any damages
  arising from the use of this software.
  
  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:
  
  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.
  
  Alex Diener adiener@sacredsoftware.net
*/

#ifndef __GAMEPLAY_STATE_VIEW_H__
#define __GAMEPLAY_STATE_VIEW_H__

typedef struct GameplayStateView GameplayStateView;

#include "convergence/DungeonRoomView.h"
#include "convergence/GameObjectView.h"
#include "convergence/GameplayCamera.h"
#include "convergence/GameStateModel.h"
#include "glgraphics/GLGraphics.h"
#include "stemobject/StemObject.h"
#include "resourcemanager/ResourceManager.h"
#include <stdint.h>

#define GameplayStateView_structContents \
	StemObject_structContents \
	\
	GameStateModel * gameStateModel; \
	GameplayCamera * camera; \
	ResourceManager * resourceManager; \
	DungeonRoomStateModel * roomState; \
	DungeonRoomView * roomView; \
	size_t objectViewCount; \
	size_t objectViewListSize; \
	GameObjectView ** objectViews; \
	GLuint vboID; \
	GLuint eboID; \
	bool playingFadeEffect; \
	uint32_t fadeEffectStartColor; \
	uint32_t fadeEffectEndColor; \
	double currentTime; \
	double fadeEffectDuration; \
	double fadeEffectStartTime; \
	bool dungeonComplete; \
	double dungeonCompletionTime; \
	\
	void (* playFadeEffect)(compat_type(GameplayStateView *) self, uint32_t startColor, uint32_t endColor, double duration); \
	void (* draw)(compat_type(GameplayStateView *) self, double deltaTime);

struct GameplayStateView {
	GameplayStateView_structContents
};

GameplayStateView * GameplayStateView_create(GameStateModel * gameStateModel, GameplayCamera * camera, ResourceManager * resourceManager);
void GameplayStateView_init(compat_type(GameplayStateView *) selfPtr, GameStateModel * gameStateModel, GameplayCamera * camera, ResourceManager * resourceManager);
void GameplayStateView_dispose(compat_type(GameplayStateView *) selfPtr);

void GameplayStateView_playFadeEffect(compat_type(GameplayStateView *) selfPtr, uint32_t startColor, uint32_t endColor, double duration);
void GameplayStateView_draw(compat_type(GameplayStateView *) selfPtr, double deltaTime);

#endif
