/*
  Copyright (c) 2011 Alex Diener
  
  This software is provided 'as-is', without any express or implied
  warranty. In no event will the authors be held liable for any damages
  arising from the use of this software.
  
  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:
  
  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.
  
  Alex Diener adiener@sacredsoftware.net
*/

#include "convergence/PlayerInputController.h"

PlayerInputController * PlayerInputController_create(compat_type(PlayerModel *) playerModelPtr, PlayerInputMap * inputMap) {
	stemobject_create_implementation(PlayerInputController, init, playerModelPtr, inputMap)
}

static bool playerDied(void * sender, Atom eventID, void * eventData, void * context) {
	PlayerInputController * self = context;
	
	self->playerModel->eventDispatcher->unregisterForEvent(self->playerModel->eventDispatcher, Atom_fromString(PLAYER_EVENT_DEATH), playerDied, self);
	self->playerModel = NULL;
	return true;
}

void PlayerInputController_init(compat_type(PlayerInputController *) selfPtr, compat_type(PlayerModel *) playerModelPtr, PlayerInputMap * inputMap) {
	PlayerInputController * self = selfPtr;
	
	InputController_init(self, inputMap);
	self->dispose = PlayerInputController_dispose;
	self->protected_ivar(triggerAction) = PlayerInputController_triggerAction;
	self->protected_ivar(releaseAction) = PlayerInputController_releaseAction;
	self->setPlayerModel = PlayerInputController_setPlayerModel;
	
	self->playerModel = playerModelPtr;
	self->playerModel->eventDispatcher->registerForEvent(self->playerModel->eventDispatcher, Atom_fromString(PLAYER_EVENT_DEATH), playerDied, self);
}

void PlayerInputController_dispose(compat_type(PlayerInputController *) selfPtr) {
	PlayerInputController * self = selfPtr;
	
	if (self->playerModel != NULL) {
		self->playerModel->eventDispatcher->unregisterForEvent(self->playerModel->eventDispatcher, Atom_fromString(PLAYER_EVENT_DEATH), playerDied, self);
	}
	InputController_dispose(self);
}

bool PlayerInputController_triggerAction(compat_type(PlayerInputController *) selfPtr, Atom actionID) {
	PlayerInputController * self = selfPtr;
	
	if (actionID == Atom_fromString("left")) {
		if (!self->leftTriggered) {
			self->leftTriggered = true;
			if (self->playerModel != NULL) {
				self->playerModel->startMovingLeft(self->playerModel);
			}
		}
		return true;
	}
	if (actionID == Atom_fromString("right")) {
		if (!self->rightTriggered) {
			self->rightTriggered = true;
			if (self->playerModel != NULL) {
				self->playerModel->startMovingRight(self->playerModel);
			}
		}
		return true;
	}
	if (actionID == Atom_fromString("down")) {
		if (!self->downTriggered) {
			self->downTriggered = true;
			if (self->playerModel != NULL) {
				self->playerModel->startMovingDown(self->playerModel);
			}
		}
		return true;
	}
	if (actionID == Atom_fromString("up")) {
		if (!self->upTriggered) {
			self->upTriggered = true;
			if (self->playerModel != NULL) {
				self->playerModel->startMovingUp(self->playerModel);
			}
		}
		return true;
	}
	if (actionID == Atom_fromString("attack")) {
		if (!self->attackTriggered) {
			self->attackTriggered = true;
			if (self->playerModel != NULL) {
				self->playerModel->startAttack(self->playerModel);
			}
		}
		return true;
	}
	if (actionID == Atom_fromString("special")) {
		if (!self->specialTriggered) {
			self->specialTriggered = true;
			if (self->playerModel != NULL) {
				self->playerModel->startSpecialAction(self->playerModel);
			}
		}
		return true;
	}
	return false;
}

bool PlayerInputController_releaseAction(compat_type(PlayerInputController *) selfPtr, Atom actionID) {
	PlayerInputController * self = selfPtr;
	
	if (actionID == Atom_fromString("left")) {
		self->leftTriggered = false;
		if (self->playerModel != NULL) {
			self->playerModel->stopMovingLeft(self->playerModel);
		}
		return true;
	}
	if (actionID == Atom_fromString("right")) {
		self->rightTriggered = false;
		if (self->playerModel != NULL) {
			self->playerModel->stopMovingRight(self->playerModel);
		}
		return true;
	}
	if (actionID == Atom_fromString("down")) {
		self->downTriggered = false;
		if (self->playerModel != NULL) {
			self->playerModel->stopMovingDown(self->playerModel);
		}
		return true;
	}
	if (actionID == Atom_fromString("up")) {
		self->upTriggered = false;
		if (self->playerModel != NULL) {
			self->playerModel->stopMovingUp(self->playerModel);
		}
		return true;
	}
	if (actionID == Atom_fromString("attack")) {
		self->attackTriggered = false;
		if (self->playerModel != NULL) {
			self->playerModel->stopAttack(self->playerModel);
		}
		return true;
	}
	if (actionID == Atom_fromString("special")) {
		self->specialTriggered = false;
		if (self->playerModel != NULL) {
			self->playerModel->stopSpecialAction(self->playerModel);
		}
		return true;
	}
	return false;
}

void PlayerInputController_setPlayerModel(compat_type(PlayerInputController *) selfPtr, compat_type(PlayerModel *) playerModelPtr) {
	PlayerInputController * self = selfPtr;
	
	self->playerModel = playerModelPtr;
	self->playerModel->eventDispatcher->registerForEvent(self->playerModel->eventDispatcher, Atom_fromString(PLAYER_EVENT_DEATH), playerDied, self);
}
