/*
  Copyright (c) 2011 Alex Diener
  
  This software is provided 'as-is', without any express or implied
  warranty. In no event will the authors be held liable for any damages
  arising from the use of this software.
  
  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:
  
  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.
  
  Alex Diener adiener@sacredsoftware.net
*/

#ifndef __RAINBOW_TRAIL_EFFECT_H__
#define __RAINBOW_TRAIL_EFFECT_H__

typedef struct RainbowTrailEffect RainbowTrailEffect;

#include "3dmath/Vector.h"
#include "convergence/GameObjectView.h"
#include "convergence/PlayerSpeedsterModel.h"
#include "glgraphics/GLIncludes.h"
#include "gltexture/GLTexture.h"
#include "resourcemanager/ResourceManager.h"

#define RAINBOW_TRAIL_EFFECT_NODE_MAX 40

struct RainbowTrailEffect_node {
	Vector2 leftVertex;
	Vector2 rightVertex;
	double time;
};

#define RainbowTrailEffect_structContents \
	GameObjectView_structContents \
	\
	ResourceManager * resourceManager; \
	PlayerSpeedsterModel * player; \
	GLTexture * texture; \
	int nodeCount; \
	struct RainbowTrailEffect_node nodes[RAINBOW_TRAIL_EFFECT_NODE_MAX]; \
	Vector2 lastPlayerPosition; \
	double currentTime; \
	GLuint vboID; \
	GLuint eboID;

struct RainbowTrailEffect {
	RainbowTrailEffect_structContents
};

RainbowTrailEffect * RainbowTrailEffect_create(compat_type(PlayerSpeedsterModel *) player, ResourceManager * resourceManager);
void RainbowTrailEffect_init(compat_type(RainbowTrailEffect *) selfPtr, compat_type(PlayerSpeedsterModel *) player, ResourceManager * resourceManager);
void RainbowTrailEffect_dispose(compat_type(RainbowTrailEffect *) selfPtr);

void RainbowTrailEffect_draw(compat_type(RainbowTrailEffect *) selfPtr, Matrix modelviewMatrix, double deltaTime);

#endif
